package com.zehong.netty.decode;

import com.zehong.entity.THeatData;
import com.zehong.service.ITHeatDataService;
import com.zehong.utils.DateTimeValueUtil;
import com.zehong.utils.HexUtils;
import com.zehong.utils.SpringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;

/**
 * @author gengd
 * 信息单元热量表数据解析
 */
public class HeatMeterDataListDecode {

    private Logger logger = LoggerFactory.getLogger(HeatMeterDataListDecode.class);

    private MsgDecode decode;

    /**
     * 累计冷量单位，单位最高位为0表示累计冷量为正；最高位为1表示累计冷量为负
     */
    private Map<String,String> cumulativeCoolUnitMap = new HashMap<>();

    {
        //焦耳
        cumulativeCoolUnitMap.put("01","J");
        //千焦
        cumulativeCoolUnitMap.put("0b","KJ");
        //兆焦
        cumulativeCoolUnitMap.put("0e","MJ");
        //G焦
        cumulativeCoolUnitMap.put("11","GJ");
        //G焦 * 100
        cumulativeCoolUnitMap.put("13","GJ*100");
        //瓦时
        cumulativeCoolUnitMap.put("02","Wh");
        //千瓦时
        cumulativeCoolUnitMap.put("05","kWh");
        //兆瓦时
        cumulativeCoolUnitMap.put("08","MWh");
        //兆瓦时 * 100
        cumulativeCoolUnitMap.put("0a","MWH*100");
    }

    /**
     * 瞬时功率单位
     */
    private Map<String,String> theCurrentThermalPowerUnitMap = new HashMap<>();

    {
        //瓦
        theCurrentThermalPowerUnitMap.put("14","W");
        //千瓦
        theCurrentThermalPowerUnitMap.put("17","kW");
        //兆瓦
        theCurrentThermalPowerUnitMap.put("1a","MW");
    }

    /**
     * 瞬时流量单位
     */
    private Map<String,String> currentFlowUnitMap = new HashMap<>();

    {
        //升/小时
        currentFlowUnitMap.put("32","L/h");
        //立方米/小时
        currentFlowUnitMap.put("35","m³/h");
    }

    /**
     * 累计流量单位
     */
    private Map<String,String> cumulativeFlowUnitMap = new HashMap<>();

    {
        //升
        cumulativeFlowUnitMap.put("29","L");
        //立方米
        cumulativeFlowUnitMap.put("2c","m³");
    }

    /**
     * 热量表数据
     */
    private String heatMeterDataList;

    /**
     * 热量表数据所占字节数
     */
    public int getHeatMeterDataListLength;

    public HeatMeterDataListDecode(MsgDecode decode){
        heatMeterDataList = decode.getMsgInfo().substring(42,decode.getMsgInfo().length());
    }

    /**
     * 解析热量表数据
     */
    public void decodeHeatMeterDataList(){
        //信息单元类型
        String ieType = this.heatMeterDataList.substring(0,2);
        //长度
        String length = this.heatMeterDataList.substring(2,6);
        //热量表数量
        String heatMeterDataCount = this.heatMeterDataList.substring(6,10);
        //热量表数据所占字节数
        this.getHeatMeterDataListLength = Integer.parseInt(heatMeterDataCount,16) * 63 + 5;
        //热量表数据
        decodeHeatMeterData(Integer.parseInt(heatMeterDataCount,16));
        logger.info("transmissionId:" + decode.getTransmissionId() + "热量表数据： 信息单元类型："+ ieType +" 长度：" + length + "热量表数量：" + heatMeterDataCount + "热量表数据：" + heatMeterDataList.substring(10,Integer.parseInt(heatMeterDataCount,16)*126));
    }

    /**
     * 解析热量表数据
     * @param count 热量表数量
     */
    private void decodeHeatMeterData(int count){
        String heatMeterDatas = heatMeterDataList.substring(10,heatMeterDataList.length());
        for(int i = 1; i < count + 1; i++){
            String heatMeterData = heatMeterDatas.substring((i-1)*126,i*126);
            saveHeatMeterData(heatMeterData);
        }
    }

    /**
     * 保存热量表信息
     * @param heatMeterData 热量表数据
     */
    private void saveHeatMeterData(String heatMeterData){
        ITHeatDataService itHeatDataService = SpringUtil.getBean(ITHeatDataService.class);
        THeatData tHeatData = decodeHeatMeterData(heatMeterData);
        itHeatDataService.insertTHeatData(tHeatData);
    }

    /**
     * 解析流量表数据
     * @param heatMeterData 流量表数据
     * @return THeatData
     */
    private THeatData decodeHeatMeterData(String heatMeterData){
        THeatData tHeatData = new THeatData();
        //厂家编号
        String manufacturersId = heatMeterData.substring(0,4);
        tHeatData.setManufacturersId(manufacturersId);
        //表类型
        String equipmentTypeValue = heatMeterData.substring(4,6);
        tHeatData.setEquipmentTypeValue(equipmentTypeValue);
        //热量表编号
        String heatMeterId = heatMeterData.substring(6,16);
        tHeatData.setHeatMeterId(heatMeterId);
        //累计冷量
        String cumulativeCool = heatMeterData.substring(18,26);
        tHeatData.setCumulativeCool(new BigDecimal(Integer.parseInt(cumulativeCool,16)/100));
        //累计冷量单位
        String cumulativeCoolUnit = heatMeterData.substring(26,28);
        if(cumulativeCool.substring(0).equals("1")){
            tHeatData.setCumulativeCoolUnit("-" + cumulativeCoolUnitMap.get(cumulativeCoolUnit));
        }else{
            tHeatData.setCumulativeCoolUnit(cumulativeCoolUnitMap.get(cumulativeCoolUnit));
        }
        //累积热量
        String cumulativeHeat = heatMeterData.substring(28,36);
        tHeatData.setCumulativeHeat(new BigDecimal(Integer.parseInt(cumulativeHeat,16)/100));
        //累计热量单位
        String cumulativeHeatUnit = heatMeterData.substring(36,38);
        if(cumulativeHeatUnit.substring(0).equals("1")){
            tHeatData.setCumulativeHeatUnit("-" + cumulativeCoolUnitMap.get(cumulativeCoolUnit));
        }else{
            tHeatData.setCumulativeHeatUnit(cumulativeCoolUnitMap.get(cumulativeCoolUnit));
        }
        //瞬时功率
        String theCurrentThermalPower = heatMeterData.substring(38,46);
        tHeatData.setTheCurrentThermalPower(new BigDecimal(Integer.parseInt(theCurrentThermalPower,16)/100));
        //瞬时功率单位
        String theCurrentThermalPowerUnit = heatMeterData.substring(46,48);
        if(theCurrentThermalPowerUnit.substring(0).equals("1")){
            tHeatData.setTheCurrentThermalPowerUnit("-" + theCurrentThermalPowerUnitMap.get(theCurrentThermalPower));
        }else{
            tHeatData.setTheCurrentThermalPowerUnit(theCurrentThermalPowerUnitMap.get(theCurrentThermalPower));
        }
        //瞬时流量
        String currentFlow = heatMeterData.substring(48,56);
        tHeatData.setCurrentFlow(new BigDecimal(Integer.parseInt(currentFlow,16)/100));
        //瞬时流量单位
        String currentFlowUnit = heatMeterData.substring(56,58);
        if(currentFlowUnit.substring(0).equals("1")){
            tHeatData.setCurrentFlowUnit("-" + currentFlowUnitMap.get(currentFlowUnit));
        }else{
            tHeatData.setCurrentFlowUnit(currentFlowUnitMap.get(currentFlowUnit));
        }
        //累计流量
        String cumulativeFlow = heatMeterData.substring(58,66);
        tHeatData.setCumulativeFlow(new BigDecimal(Integer.parseInt(cumulativeFlow,16)/100));
        //累计流量单位
        String cumulativeFlowUnit = heatMeterData.substring(66,68);
        if(cumulativeFlowUnit.substring(0).equals("1")){
            tHeatData.setCumulativeFlowUnit("-" + currentFlowUnitMap.get(cumulativeFlowUnit));
        }else{
            tHeatData.setCumulativeFlowUnit(cumulativeFlowUnitMap.get(cumulativeFlowUnit));
        }
        //当前供水温度
        String theCurrentWaterTemperature = heatMeterData.substring(68,72);
        tHeatData.setTheCurrentWaterTemperature(new BigDecimal(Integer.parseInt(theCurrentWaterTemperature,16)/100));
        //当前回水温度
        String currentReturnTemperature = heatMeterData.substring(72,76);
        tHeatData.setCurrentReturnTemperature(new BigDecimal(Integer.parseInt(currentReturnTemperature,16)/100));
        //累积工作时间 单位小时
        String cumulativeWorkingTime = heatMeterData.substring(76,84);
        tHeatData.setCumulativeWorkingTime(String.valueOf(Integer.parseInt(cumulativeWorkingTime,16)));
        //热量表实时时间
        String dateTimeValue = heatMeterData.substring(84,98);
        try {
            tHeatData.setDateTimeValue(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse( DateTimeValueUtil.getDateTimeStr(dateTimeValue)));
        } catch (ParseException e) {
            logger.error("热量表实时时间获取失败");
        }
        //回水累计流量
        String cumulativeReturnFlow = heatMeterData.substring(98,106);
        tHeatData.setCumulativeReturnFlow(new BigDecimal(Integer.parseInt(cumulativeReturnFlow,16)/100));
        //回水累计流量单位
        String cumulativeReturnFlowUnit = heatMeterData.substring(106,108);
        if(cumulativeReturnFlowUnit.substring(0).equals("1")){
            tHeatData.setCumulativeReturnFlowUnit("-" + currentFlowUnitMap.get(cumulativeReturnFlowUnit));
        }else{
            tHeatData.setCumulativeReturnFlowUnit(cumulativeFlowUnitMap.get(cumulativeReturnFlowUnit));
        }
        //累计流量差
        String cumulativeFlowDifference = heatMeterData.substring(108,116);
        tHeatData.setCumulativeFlowDifference(new BigDecimal(Integer.parseInt(cumulativeFlowDifference,16)/100));
        //累计流量差单位
        String cumulativeFlowDifferenceUnit = heatMeterData.substring(116,118);
        if(cumulativeFlowDifferenceUnit.substring(0).equals("1")){
            tHeatData.setCumulativeFlowDifferenceUnit("-" + currentFlowUnitMap.get(cumulativeFlowDifferenceUnit));
        }else{
            tHeatData.setCumulativeFlowDifferenceUnit(cumulativeFlowUnitMap.get(cumulativeFlowDifferenceUnit));
        }
        //有效标识
        String returnFlowFlag = heatMeterData.substring(118,122);
        tHeatData.setReturnFlowFlag(String.valueOf(Integer.parseInt(returnFlowFlag,16)));
        //stStutas
        String stStatus = heatMeterData.substring(122,124);
        tHeatData.setStStatus(HexUtils.hexToBinary(stStatus));
        logger.info("transmissionId:" + decode.getTransmissionId() + "设备数据：" + tHeatData.toString());
        return tHeatData;
    }

}
