package com.zehong.netty.handler;

import cn.hutool.core.date.DateUtil;
import com.zehong.enums.MessageTypeEnum;
import com.zehong.netty.decode.InformationDecode;
import com.zehong.netty.decode.MsgDecode;
import com.zehong.utils.HexUtils;
import com.zehong.utils.RedisUtil;
import com.zehong.utils.SpringUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.socket.SocketChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.List;
import java.util.zip.CRC32;

/**
 * @author gengd
 * 物联网 开启检测端口 并写入数据库
 */
@Component
@Sharable
public class TcpServerHandler extends ChannelInboundHandlerAdapter {

    private Logger logger = LoggerFactory.getLogger(TcpServerHandler.class);

    @Override
    public void channelActive(ChannelHandlerContext ctx) {
        SocketChannel channel = (SocketChannel) ctx.channel();
        logger.info("链接报告开始");
        logger.info("链接报告信息：有一客户端链接到本服务端");
        logger.info("链接报告IP:{}", channel.localAddress().getHostString());
        logger.info("链接报告Port:{}", channel.localAddress().getPort());
        logger.info("链接报告完毕");
        //通知客户端链接建立成功
        String str = "通知客户端链接建立成功" + " " + new Date() + " " + channel.localAddress().getHostString() + "\r\n";
        ctx.writeAndFlush(str);
    }

    /**
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx){
        logger.info("客户端断开链接{}", ctx.channel().localAddress().toString());
    }
    /**
     * 获取数据
     * @param ctx 上下文
     * @param msg 获取的数据
     * @throws UnsupportedEncodingException
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg){
        ByteBuf buf = (ByteBuf)msg;
        byte [] msgBytes = new byte[buf.readableBytes()];
        buf.readBytes(msgBytes);
        //16进制消息字符串
        String msgStr = HexUtils.bytesToHex(msgBytes);
        logger.info("tcp上传数据为：" + msgStr);
        //解析上传数据
        MsgDecode decode = new MsgDecode(msgStr);
        if(!MessageTypeEnum.READ_DATA_RESPONSE.value.equals(decode.getMessageType())){
            logger.info("此消息类型不做处理！");
            return;
        }
        if(!decode.checkCRC32Code()){
            logger.error("上传数据校验失败！");
            return;
        }
        //解析信息单元
        InformationDecode info = new InformationDecode(decode);
        info.infoMsgDecode();
        if(judgeIsLastPackage(decode,info.totalPackageCount)){
            ctx.writeAndFlush(confirmInformation(decode));
        }
    }

    /**
     *  判断是否为最后一次分包信息
     * @param decode 上传数据
     * @param totalPackageCount 分包总数
     * @return boolean
     */
    private boolean judgeIsLastPackage(MsgDecode decode,String totalPackageCount){
        RedisUtil redisUtil = SpringUtil.getBean(RedisUtil.class);
        List<String> cashInfo = (List<String>) redisUtil.get(decode.getTransmissionId());
        //如接受到所有分包信息，移除分包缓存
        if( Integer.parseInt(totalPackageCount,16) == cashInfo.size()){
            //确认消息
            //confirmInformation();
            //移除分包缓存
            redisUtil.del(decode.getTransmissionId());
            return true;
        }
        return false;
    }

    /**
     * 确认成功接收消息
     */
    private String confirmInformation(MsgDecode decode){
        String head = decode.getMsgHead();
        String tail = decode.getMsgTail();
        String dateTime = getDateTimeHex();
        CRC32 crc32 = new CRC32();
        crc32.update((head.substring(8,head.length())+dateTime).getBytes());
        String checkCode = Long.toHexString(crc32.getValue());
        StringBuilder reqMsg = new StringBuilder(head);
        reqMsg.append(dateTime)
                .append(checkCode)
                .append(tail.substring(8,12));
        logger.info("transmissionId:" + decode.getTransmissionId() + "消息确认服务器回复消息：" + reqMsg.toString());
        return reqMsg.toString();
    }

    /**
     * 拼接信息单元
     * @return String
     */
    private static String getDateTimeHex(){
        String now = DateUtil.now();
        String[] date =now.split(" ");
        //年月日
        String[] yearDate = date[0].split("-");
        //时分秒
        String[] time = date[1].split(":");
        System.out.println("年：" + HexUtils.bytesToHex(yearDate[0].substring(0,2).getBytes()) + " 时：" + time.toString());
        StringBuilder dateTime = new StringBuilder("");
        dateTime.append(String.format("%02x",Integer.parseInt(yearDate[0].substring(0,2))))
                .append(String.format("%02x",Integer.parseInt(yearDate[0].substring(2,4))))
                .append(String.format("%02x",Integer.parseInt(yearDate[1])))
                .append(String.format("%02x",Integer.parseInt(yearDate[2])))
                .append(String.format("%02x",Integer.parseInt(time[0])))
                .append(String.format("%02x",Integer.parseInt(time[1])))
                .append(String.format("%02x",Integer.parseInt(time[2])));
        return dateTime.toString();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        logger.error("tcp连接报错",cause);
        cause.printStackTrace();
        ctx.close();
    }
}
